@@ -25,8 +25,9 @@ G_BEGIN_DECLS
 
 #include "oobs-list.h"
 
-OobsList*   _oobs_list_new   (GType contained_type);
-void        _oobs_list_set_locked (OobsList *list, gboolean locked);
+OobsList*   _oobs_list_new        (const GType contained_type);
+void        _oobs_list_set_locked (OobsList   *list,
+                                   gboolean    locked);
 
 
 G_END_DECLS
